@@ -51,6 +51,7 @@ struct sim_data {
 	GAtChat *chat;
 	unsigned int vendor;
 	guint ready_id;
+	struct at_util_sim_state_query *sim_state_query;
 };
 
 static const char *crsm_prefix[] = { "+CRSM:", NULL };
@@ -59,9 +60,11 @@ static const char *clck_prefix[] = { "+CLCK:", NULL };
 static const char *huawei_cpin_prefix[] = { "^CPIN:", NULL };
 static const char *xpincnt_prefix[] = { "+XPINCNT:", NULL };
 static const char *zpinpuk_prefix[] = { "+ZPINPUK:", NULL };
+static const char *pinnum_prefix[] = { "%PINNUM:", NULL };
 static const char *oercn_prefix[] = { "_OERCN:", NULL };
 static const char *cpinr_prefixes[] = { "+CPINR:", "+CPINRE:", NULL };
 static const char *epin_prefix[] = { "*EPIN:", NULL };
+static const char *spic_prefix[] = { "+SPIC:", NULL };
 static const char *none_prefix[] = { NULL };
 
 static void at_crsm_info_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -111,8 +114,7 @@ static void at_crsm_info_cb(gboolean ok, GAtResult *result, gpointer user_data)
 						&str, access, NULL);
 
 		file_status = EF_STATUS_VALID;
-	}
-	else
+	} else
 		ok = sim_parse_2g_get_response(response, len, &flen, &rlen,
 						&str, access, &file_status);
 
@@ -171,7 +173,7 @@ static void at_sim_read_info(struct ofono_sim *sim, int fileid,
 }
 
 static void at_crsm_read_cb(gboolean ok, GAtResult *result,
-		gpointer user_data)
+				gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	GAtResultIter iter;
@@ -259,7 +261,7 @@ static void at_sim_read_record(struct ofono_sim *sim, int fileid,
 }
 
 static void at_crsm_update_cb(gboolean ok, GAtResult *result,
-		gpointer user_data)
+				gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	GAtResultIter iter;
@@ -297,25 +299,46 @@ static void at_crsm_update_cb(gboolean ok, GAtResult *result,
 	cb(&error, cbd->data);
 }
 
-static void at_sim_update_binary(struct ofono_sim *sim, int fileid,
-					int start, int length,
-					const unsigned char *value,
-					ofono_sim_write_cb_t cb, void *data)
+static void at_sim_update_file(struct ofono_sim *sim, int cmd, int fileid,
+				int p1, int p2, int p3,
+				const unsigned char *value,
+				ofono_sim_write_cb_t cb, void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	struct cb_data *cbd = cb_data_new(cb, data);
-	char *buf = g_try_new(char, 36 + length * 2);
+	char *buf;
+	gboolean quote = FALSE;
 	int len, ret;
+	int size = 36 + p3 * 2;
 
+	DBG("");
+
+	/* Add quotes */
+	switch (sd->vendor) {
+	case OFONO_VENDOR_MBM:
+	case OFONO_VENDOR_ZTE:
+	case OFONO_VENDOR_HUAWEI:
+	case OFONO_VENDOR_SPEEDUP:
+		quote = TRUE;
+		size += 2;
+		break;
+	}
+
+	buf = g_try_new(char, size);
 	if (buf == NULL)
 		goto error;
 
-	len = sprintf(buf, "AT+CRSM=214,%i,%i,%i,%i,", fileid,
-			start >> 8, start & 0xff, length);
+	len = sprintf(buf, "AT+CRSM=%i,%i,%i,%i,%i,", cmd, fileid,p1, p2, p3);
 
-	for (; length; length--)
+	if (quote)
+		buf[len++] = '\"';
+
+	for (; p3; p3--)
 		len += sprintf(buf + len, "%02hhX", *value++);
 
+	if (quote)
+		buf[len++] = '\"';
+
 	ret = g_at_chat_send(sd->chat, buf, crsm_prefix,
 				at_crsm_update_cb, cbd, g_free);
 
@@ -330,90 +353,29 @@ error:
 	CALLBACK_WITH_FAILURE(cb, data);
 }
 
+static void at_sim_update_binary(struct ofono_sim *sim, int fileid,
+					int start, int length,
+					const unsigned char *value,
+					ofono_sim_write_cb_t cb, void *data)
+{
+	at_sim_update_file(sim, 214, fileid, start >> 8, start & 0xff,
+				length, value, cb, data);
+}
+
 static void at_sim_update_record(struct ofono_sim *sim, int fileid,
 					int record, int length,
 					const unsigned char *value,
 					ofono_sim_write_cb_t cb, void *data)
 {
-	struct sim_data *sd = ofono_sim_get_data(sim);
-	struct cb_data *cbd = cb_data_new(cb, data);
-	char *buf;
-	int len, ret;
-	int size = 36 + length * 2;
-
-	if (sd->vendor == OFONO_VENDOR_MBM)
-		size += 2; /*Add quotes*/
-
-	buf = g_try_new(char, size);
-	if (buf == NULL)
-		goto error;
-
-	len = sprintf(buf, "AT+CRSM=220,%i,%i,4,%i,", fileid,
-			record, length);
-
-	if (sd->vendor == OFONO_VENDOR_MBM)
-		len += sprintf(buf + len, "\"");
-
-	for (; length; length--)
-		len += sprintf(buf + len, "%02hhX", *value++);
-
-	if (sd->vendor == OFONO_VENDOR_MBM)
-		sprintf(buf + len, "\"");
-
-	ret = g_at_chat_send(sd->chat, buf, crsm_prefix,
-				at_crsm_update_cb, cbd, g_free);
-
-	g_free(buf);
-
-	if (ret > 0)
-		return;
-
-error:
-	g_free(cbd);
-
-	CALLBACK_WITH_FAILURE(cb, data);
+	at_sim_update_file(sim, 220, fileid, record, 4, length,
+				value, cb, data);
 }
 
 static void at_sim_update_cyclic(struct ofono_sim *sim, int fileid,
 					int length, const unsigned char *value,
 					ofono_sim_write_cb_t cb, void *data)
 {
-	struct sim_data *sd = ofono_sim_get_data(sim);
-	struct cb_data *cbd = cb_data_new(cb, data);
-	char *buf;
-	int len, ret;
-	int size = 36 + length * 2;
-
-	if (sd->vendor == OFONO_VENDOR_MBM)
-		size += 2; /* Add quotes */
-
-	buf = g_try_new(char, size);
-	if (buf == NULL)
-		goto error;
-
-	len = sprintf(buf, "AT+CRSM=220,%i,0,3,%i,", fileid, length);
-
-	if (sd->vendor == OFONO_VENDOR_MBM)
-		len += sprintf(buf + len, "\"");
-
-	for (; length; length--)
-		len += sprintf(buf + len, "%02hhX", *value++);
-
-	if (sd->vendor == OFONO_VENDOR_MBM)
-		sprintf(buf + len, "\"");
-
-	ret = g_at_chat_send(sd->chat, buf, crsm_prefix,
-				at_crsm_update_cb, cbd, g_free);
-
-	g_free(buf);
-
-	if (ret > 0)
-		return;
-
-error:
-	g_free(cbd);
-
-	CALLBACK_WITH_FAILURE(cb, data);
+	at_sim_update_file(sim, 220, fileid, 0, 3, length, value, cb, data);
 }
 
 static void at_cimi_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -445,7 +407,7 @@ static void at_cimi_cb(gboolean ok, GAtResult *result, gpointer user_data)
 }
 
 static void at_read_imsi(struct ofono_sim *sim, ofono_sim_imsi_cb_t cb,
-			void *data)
+				void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	struct cb_data *cbd = cb_data_new(cb, data);
@@ -544,6 +506,46 @@ error:
 	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
 }
 
+static void pinnum_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+	static enum ofono_sim_password_type password_types[] = {
+		OFONO_SIM_PASSWORD_SIM_PIN,
+		OFONO_SIM_PASSWORD_SIM_PUK,
+		OFONO_SIM_PASSWORD_SIM_PIN2,
+		OFONO_SIM_PASSWORD_SIM_PUK2,
+	};
+
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "%PINNUM:"))
+		goto error;
+
+	BUILD_PIN_RETRIES_ARRAY(password_types, ARRAY_SIZE(password_types),
+				retries);
+
+	cb(&error, retries, cbd->data);
+
+	return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
+}
+
 static void zpinpuk_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
@@ -800,6 +802,45 @@ static void at_cpinr_cb(gboolean ok, GAtResult *result, gpointer user_data)
 	cb(&error, retries, cbd->data);
 }
 
+static void at_spic_cb(gboolean ok, GAtResult *result, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	ofono_sim_pin_retries_cb_t cb = cbd->cb;
+	const char *final = g_at_result_final_response(result);
+	GAtResultIter iter;
+	struct ofono_error error;
+	int retries[OFONO_SIM_PASSWORD_INVALID];
+	size_t i;
+	static enum ofono_sim_password_type password_types[] = {
+		OFONO_SIM_PASSWORD_SIM_PIN,
+		OFONO_SIM_PASSWORD_SIM_PUK,
+		OFONO_SIM_PASSWORD_SIM_PIN2,
+		OFONO_SIM_PASSWORD_SIM_PUK2,
+	};
+
+	decode_at_error(&error, final);
+
+	if (!ok) {
+		cb(&error, NULL, cbd->data);
+		return;
+	}
+
+	g_at_result_iter_init(&iter, result);
+
+	if (!g_at_result_iter_next(&iter, "+SPIC:"))
+		goto error;
+
+	BUILD_PIN_RETRIES_ARRAY(password_types, ARRAY_SIZE(password_types),
+				retries);
+
+	cb(&error, retries, cbd->data);
+
+	return;
+
+error:
+	CALLBACK_WITH_FAILURE(cb, NULL, cbd->data);
+}
+
 static void at_pin_retries_query(struct ofono_sim *sim,
 					ofono_sim_pin_retries_cb_t cb,
 					void *data)
@@ -830,6 +871,11 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 					huawei_cpin_cb, cbd, g_free) > 0)
 			return;
 		break;
+	case OFONO_VENDOR_ICERA:
+		if (g_at_chat_send(sd->chat, "AT%PINNUM?", pinnum_prefix,
+					pinnum_cb, cbd, g_free) > 0)
+			return;
+		break;
 	case OFONO_VENDOR_ZTE:
 		if (g_at_chat_send(sd->chat, "AT+ZPINPUK=?", zpinpuk_prefix,
 					zpinpuk_cb, cbd, g_free) > 0)
@@ -840,6 +886,11 @@ static void at_pin_retries_query(struct ofono_sim *sim,
 					at_epin_cb, cbd, g_free) > 0)
 			return;
 		break;
+	case OFONO_VENDOR_SIMCOM:
+		if (g_at_chat_send(sd->chat, "AT+SPIC", spic_prefix,
+					at_spic_cb, cbd, g_free) > 0)
+			return;
+		break;
 	default:
 		if (g_at_chat_send(sd->chat, "AT+CPINR", cpinr_prefixes,
 					at_cpinr_cb, cbd, g_free) > 0)
@@ -875,7 +926,8 @@ static void at_cpin_cb(gboolean ok, GAtResult *result, gpointer user_data)
 		return;
 	}
 
-	if (sd->vendor == OFONO_VENDOR_WAVECOM) {
+	if (sd->vendor == OFONO_VENDOR_WAVECOM ||
+			sd->vendor == OFONO_VENDOR_WAVECOM_Q2XXX) {
 		/* +CPIN: <pin> */
 		pin_required = final + 7;
 	} else {
@@ -968,6 +1020,21 @@ static void at_epev_notify(GAtResult *result, gpointer user_data)
 	sd->ready_id = 0;
 }
 
+static void sim_state_cb(gboolean present, gpointer user_data)
+{
+	struct cb_data *cbd = user_data;
+	struct sim_data *sd = cbd->user;
+	ofono_sim_lock_unlock_cb_t cb = cbd->cb;
+
+	at_util_sim_state_query_free(sd->sim_state_query);
+	sd->sim_state_query = NULL;
+
+	if (present == 1)
+		CALLBACK_WITH_SUCCESS(cb, cbd->data);
+	else
+		CALLBACK_WITH_FAILURE(cb, cbd->data);
+}
+
 static void at_pin_send_cb(gboolean ok, GAtResult *result,
 				gpointer user_data)
 {
@@ -1002,6 +1069,16 @@ static void at_pin_send_cb(gboolean ok, GAtResult *result,
 							at_epev_notify,
 							FALSE, cbd, g_free);
 		return;
+	case OFONO_VENDOR_ZTE:
+		/*
+		 * On ZTE modems, after pin is entered, SIM state is checked
+		 * by polling CPIN as their modem doesn't provide unsolicited
+		 * notification of SIM readiness.
+		 */
+		sd->sim_state_query = at_util_sim_state_query_new(sd->chat,
+						2, 20, sim_state_cb, cbd,
+						g_free);
+		return;
 	}
 
 done:
@@ -1116,9 +1193,9 @@ error:
 }
 
 static void at_change_passwd(struct ofono_sim *sim,
-			enum ofono_sim_password_type passwd_type,
-			const char *old_passwd, const char *new_passwd,
-			ofono_sim_lock_unlock_cb_t cb, void *data)
+				enum ofono_sim_password_type passwd_type,
+				const char *old_passwd, const char *new_passwd,
+				ofono_sim_lock_unlock_cb_t cb, void *data)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 	struct cb_data *cbd = cb_data_new(cb, data);
@@ -1148,7 +1225,7 @@ error:
 }
 
 static void at_lock_status_cb(gboolean ok, GAtResult *result,
-		gpointer user_data)
+				gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
 	GAtResultIter iter;
@@ -1228,6 +1305,9 @@ static int at_sim_probe(struct ofono_sim *sim, unsigned int vendor,
 	case OFONO_VENDOR_MBM:
 		g_at_chat_send(sd->chat, "AT*EPEE=1", NULL, NULL, NULL, NULL);
 		break;
+	case OFONO_VENDOR_WAVECOM_Q2XXX:
+		g_at_chat_add_terminator(chat, "+CPIN: READY", -1, TRUE);
+		break;
 	default:
 		break;
 	}
@@ -1242,6 +1322,9 @@ static void at_sim_remove(struct ofono_sim *sim)
 {
 	struct sim_data *sd = ofono_sim_get_data(sim);
 
+	/* Cleanup potential SIM state polling */
+	at_util_sim_state_query_free(sd->sim_state_query);
+
 	ofono_sim_set_data(sim, NULL);
 
 	g_at_chat_unref(sd->chat);
