@@ -119,6 +119,8 @@ static const char *default_en_list[] = { "911", "112", NULL };
 static const char *default_en_list_no_sim[] = { "119", "118", "999", "110",
 						"08", "000", NULL };
 
+static void send_ciev_after_swap_callback(const struct ofono_error *error,
+								void *data);
 static void generic_callback(const struct ofono_error *error, void *data);
 static void hangup_all_active(const struct ofono_error *error, void *data);
 static void multirelease_callback(const struct ofono_error *err, void *data);
@@ -665,20 +667,22 @@ static DBusMessage *voicecall_answer(DBusConnection *conn,
 	return NULL;
 }
 
-static GDBusMethodTable voicecall_methods[] = {
-	{ "GetProperties",  "",    "a{sv}",   voicecall_get_properties },
-	{ "Deflect",        "s",   "",        voicecall_deflect,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "Hangup",         "",    "",        voicecall_hangup,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "Answer",         "",    "",        voicecall_answer,
-						G_DBUS_METHOD_FLAG_ASYNC },
+static const GDBusMethodTable voicecall_methods[] = {
+	{ GDBUS_METHOD("GetProperties",
+				NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+				voicecall_get_properties) },
+	{ GDBUS_ASYNC_METHOD("Deflect", GDBUS_ARGS({ "number", "s" }), NULL,
+							voicecall_deflect) },
+	{ GDBUS_ASYNC_METHOD("Hangup", NULL, NULL, voicecall_hangup) },
+	{ GDBUS_ASYNC_METHOD("Answer", NULL, NULL, voicecall_answer) },
 	{ }
 };
 
-static GDBusSignalTable voicecall_signals[] = {
-	{ "PropertyChanged",	"sv" },
-	{ "DisconnectReason",	"s" },
+static const GDBusSignalTable voicecall_signals[] = {
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
+	{ GDBUS_SIGNAL("DisconnectReason",
+					GDBUS_ARGS({ "reason", "s" })) },
 	{ }
 };
 
@@ -746,6 +750,17 @@ static void voicecall_emit_multiparty(struct voicecall *call, gboolean mpty)
 						&val);
 }
 
+static void emulator_set_indicator_forced(struct ofono_voicecall *vc,
+						const char *name, int value)
+{
+	struct ofono_modem *modem = __ofono_atom_get_modem(vc->atom);
+	struct ofono_emulator *em;
+
+	em = __ofono_atom_find(OFONO_ATOM_TYPE_EMULATOR_HFP, modem);
+	if (em)
+		__ofono_emulator_set_indicator_forced(em, name, value);
+}
+
 static void emulator_call_status_cb(struct ofono_atom *atom, void *data)
 {
 	struct ofono_emulator *em = __ofono_atom_get_data(atom);
@@ -779,6 +794,8 @@ static void notify_emulator_call_status(struct ofono_voicecall *vc)
 	unsigned int non_mpty = 0;
 	gboolean multiparty = FALSE;
 	gboolean held = FALSE;
+	unsigned int non_mpty_held = 0;
+	gboolean multiparty_held = FALSE;
 	gboolean incoming = FALSE;
 	gboolean dialing = FALSE;
 	gboolean alerting = FALSE;
@@ -805,6 +822,12 @@ static void notify_emulator_call_status(struct ofono_voicecall *vc)
 
 		case CALL_STATUS_HELD:
 			held = TRUE;
+			if (g_slist_find_custom(vc->multiparty_list,
+						GINT_TO_POINTER(v->call->id),
+						call_compare_by_id))
+				multiparty_held = TRUE;
+			else
+				non_mpty_held++;
 			break;
 
 		case CALL_STATUS_DIALING:
@@ -825,6 +848,28 @@ static void notify_emulator_call_status(struct ofono_voicecall *vc)
 		}
 	}
 
+	/*
+	 * Perform some basic sanity checks for transitionary states;
+	 * if a transitionary state is detected, then ignore it.  The call
+	 * indicators will be updated properly in the follow-on calls to
+	 * this function once the final state has been reached
+	 */
+
+	if (incoming && (held || call))
+		return;
+
+	if (waiting && (held == FALSE && call == FALSE))
+		return;
+
+	if (non_mpty > 1 || (non_mpty && multiparty))
+		return;
+
+	if (non_mpty_held > 1 || (non_mpty_held && multiparty_held))
+		return;
+
+	if (multiparty && multiparty_held)
+		return;
+
 	data.status = call || held ? OFONO_EMULATOR_CALL_ACTIVE :
 					OFONO_EMULATOR_CALL_INACTIVE;
 
@@ -851,18 +896,6 @@ static void notify_emulator_call_status(struct ofono_voicecall *vc)
 	if (held)
 		data.status = call ? OFONO_EMULATOR_CALLHELD_MULTIPLE :
 					OFONO_EMULATOR_CALLHELD_ON_HOLD;
-	else if (non_mpty > 1 || (non_mpty && multiparty))
-		/*
-		 * After call swap, it is possible that all calls move
-		 * temporarily to active state (depending on call state update
-		 * order), generating an update of callheld indicator to 0.
-		 * This will fail PTS test TP/TWC/BV-03-I.
-		 *
-		 * So, in case of multiple active calls, or an active call with
-		 * an active mutiparty call, force update of callheld indicator
-		 * to 2 (intermediate state allowed).
-		 */
-		data.status = OFONO_EMULATOR_CALLHELD_ON_HOLD;
 	else
 		data.status = OFONO_EMULATOR_CALLHELD_NONE;
 
@@ -1597,13 +1630,19 @@ static DBusMessage *manager_swap_without_accept(DBusConnection *conn,
 						DBusMessage *msg, void *data)
 {
 	struct ofono_voicecall *vc = data;
+	ofono_voicecall_cb_t cb;
 
 	if (vc->pending || vc->dial_req || vc->pending_em)
 		return __ofono_error_busy(msg);
 
 	vc->pending = dbus_message_ref(msg);
 
-	vc->driver->swap_without_accept(vc, generic_callback, vc);
+	if (voicecalls_have_active(vc) && voicecalls_have_held(vc))
+		cb = send_ciev_after_swap_callback;
+	else
+		cb = generic_callback;
+
+	vc->driver->swap_without_accept(vc, cb, vc);
 
 	return NULL;
 }
@@ -1613,6 +1652,7 @@ static DBusMessage *manager_swap_calls(DBusConnection *conn,
 					DBusMessage *msg, void *data)
 {
 	struct ofono_voicecall *vc = data;
+	ofono_voicecall_cb_t cb;
 
 	if (vc->driver->swap_without_accept)
 		return manager_swap_without_accept(conn, msg, data);
@@ -1628,7 +1668,12 @@ static DBusMessage *manager_swap_calls(DBusConnection *conn,
 
 	vc->pending = dbus_message_ref(msg);
 
-	vc->driver->hold_all_active(vc, generic_callback, vc);
+	if (voicecalls_have_active(vc) && voicecalls_have_held(vc))
+		cb = send_ciev_after_swap_callback;
+	else
+		cb = generic_callback;
+
+	vc->driver->hold_all_active(vc, cb, vc);
 
 	return NULL;
 }
@@ -2074,38 +2119,47 @@ static DBusMessage *manager_get_calls(DBusConnection *conn,
 	return reply;
 }
 
-static GDBusMethodTable manager_methods[] = {
-	{ "GetProperties",     "",    "a{sv}",      manager_get_properties },
-	{ "Dial",              "ss",  "o",          manager_dial,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "Transfer",          "",    "",           manager_transfer,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SwapCalls",         "",    "",           manager_swap_calls,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "ReleaseAndAnswer",  "",    "",           manager_release_and_answer,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "HoldAndAnswer",     "",    "",           manager_hold_and_answer,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "HangupAll",         "",    "",           manager_hangup_all,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "PrivateChat",       "o",   "ao",         multiparty_private_chat,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "CreateMultiparty",  "",    "ao",         multiparty_create,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "HangupMultiparty",  "",    "",           multiparty_hangup,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "SendTones",         "s",   "",           manager_tone,
-						G_DBUS_METHOD_FLAG_ASYNC },
-	{ "GetCalls",          "",    "a(oa{sv})",  manager_get_calls },
+static const GDBusMethodTable manager_methods[] = {
+	{ GDBUS_METHOD("GetProperties",
+			NULL, GDBUS_ARGS({ "properties", "a{sv}" }),
+			manager_get_properties) },
+	{ GDBUS_ASYNC_METHOD("Dial",
+		GDBUS_ARGS({ "number", "s" }, { "hide_callerid", "s" }),
+		GDBUS_ARGS({ "path", "o" }),
+		manager_dial) },
+	{ GDBUS_ASYNC_METHOD("Transfer", NULL, NULL, manager_transfer) },
+	{ GDBUS_ASYNC_METHOD("SwapCalls",  NULL, NULL, manager_swap_calls) },
+	{ GDBUS_ASYNC_METHOD("ReleaseAndAnswer", NULL, NULL,
+						manager_release_and_answer) },
+	{ GDBUS_ASYNC_METHOD("HoldAndAnswer", NULL, NULL,
+						manager_hold_and_answer) },
+	{ GDBUS_ASYNC_METHOD("HangupAll", NULL, NULL,
+						manager_hangup_all) },
+	{ GDBUS_ASYNC_METHOD("PrivateChat", GDBUS_ARGS({ "call", "o" }),
+						GDBUS_ARGS({ "calls", "ao" }),
+						multiparty_private_chat) },
+	{ GDBUS_ASYNC_METHOD("CreateMultiparty",
+					NULL, GDBUS_ARGS({ "calls", "o" }),
+					multiparty_create) },
+	{ GDBUS_ASYNC_METHOD("HangupMultiparty", NULL, NULL,
+							multiparty_hangup) },
+	{ GDBUS_ASYNC_METHOD("SendTones",
+				GDBUS_ARGS({ "SendTones", "s" }), NULL,
+				manager_tone) },
+	{ GDBUS_METHOD("GetCalls",
+		NULL, GDBUS_ARGS({ "calls_with_properties", "a(oa{sv})" }),
+		manager_get_calls) },
 	{ }
 };
 
-static GDBusSignalTable manager_signals[] = {
-	{ "Forwarded",		"s" },
-	{ "BarringActive",	"s" },
-	{ "PropertyChanged",	"sv" },
-	{ "CallAdded",		"oa{sv}" },
-	{ "CallRemoved",	"o" },
+static const GDBusSignalTable manager_signals[] = {
+	{ GDBUS_SIGNAL("Forwarded", GDBUS_ARGS({ "type", "s" })) },
+	{ GDBUS_SIGNAL("BarringActive", GDBUS_ARGS({ "type", "s" })) },
+	{ GDBUS_SIGNAL("PropertyChanged",
+			GDBUS_ARGS({ "name", "s" }, { "value", "v" })) },
+	{ GDBUS_SIGNAL("CallAdded",
+		GDBUS_ARGS({ "path", "o" }, { "properties", "a{sv}" })) },
+	{ GDBUS_SIGNAL("CallRemoved", GDBUS_ARGS({ "path", "o"})) },
 	{ }
 };
 
@@ -2282,6 +2336,26 @@ error:
 		g_free(v);
 }
 
+static void send_ciev_after_swap_callback(const struct ofono_error *error,
+								void *data)
+{
+	struct ofono_voicecall *vc = data;
+	DBusMessage *reply;
+
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR)
+		DBG("command failed with error: %s",
+				telephony_error_to_str(error));
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR) {
+		reply = dbus_message_new_method_return(vc->pending);
+		emulator_set_indicator_forced(vc, OFONO_EMULATOR_IND_CALLHELD,
+					OFONO_EMULATOR_CALLHELD_MULTIPLE);
+	} else
+		reply = __ofono_error_failed(vc->pending);
+
+	__ofono_dbus_pending_reply(&vc->pending, reply);
+}
+
 static void generic_callback(const struct ofono_error *error, void *data)
 {
 	struct ofono_voicecall *vc = data;
@@ -2783,6 +2857,23 @@ static void sim_watch(struct ofono_atom *atom,
 	sim_state_watch(ofono_sim_get_state(sim), vc);
 }
 
+static void emulator_send_ciev_after_swap_cb(const struct ofono_error *error,
+								void *data)
+{
+	struct ofono_voicecall *vc = data;
+
+	if (vc->pending_em == NULL)
+		return;
+
+	ofono_emulator_send_final(vc->pending_em, error);
+
+	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
+		emulator_set_indicator_forced(vc, OFONO_EMULATOR_IND_CALLHELD,
+					OFONO_EMULATOR_CALLHELD_MULTIPLE);
+
+	vc->pending_em = NULL;
+}
+
 static void emulator_generic_cb(const struct ofono_error *error, void *data)
 {
 	struct ofono_voicecall *vc = data;
@@ -2931,7 +3022,8 @@ static void emulator_chup_cb(struct ofono_emulator *em,
 			goto done;
 		}
 
-		if (voicecalls_have_active(vc) == FALSE)
+		if (voicecalls_have_active(vc) == FALSE &&
+				voicecalls_have_incoming(vc) == FALSE)
 			goto fail;
 
 		vc->pending_em = em;
@@ -3020,6 +3112,7 @@ static void emulator_chld_cb(struct ofono_emulator *em,
 	char buf[64];
 	char *info;
 	int chld;
+	ofono_voicecall_cb_t cb;
 
 	result.error = 0;
 
@@ -3061,9 +3154,14 @@ static void emulator_chld_cb(struct ofono_emulator *em,
 			if (vc->driver->hold_all_active == NULL)
 				goto fail;
 
+			if (voicecalls_have_active(vc) &&
+					voicecalls_have_held(vc))
+				cb = emulator_send_ciev_after_swap_cb;
+			else
+				cb = emulator_generic_cb;
+
 			vc->pending_em = em;
-			vc->driver->hold_all_active(vc,
-					emulator_generic_cb, vc);
+			vc->driver->hold_all_active(vc, cb, vc);
 			return;
 		case 3:
 			if (vc->driver->create_multiparty == NULL)
